<?php

namespace Nest\Events;

use Nest\Exceptions\InvalidTypeException;

trait HasEvents
{
	/**
	 * Events manager instance.
	 * @var EventsManager
	 */
	private EventsManager $eventsManager;

	/**
	 * Get the current instance of the events manager.
	 * @return EventsManager - The events manager.
	 */
	protected function getEventsManager(): EventsManager
	{
		// If there is no instance, initializing one.
		if (empty($this->eventsManager)) $this->eventsManager = new EventsManager();

		return $this->eventsManager;
	}

	/**
	 * Add a listener for the given event type.
	 * @template T of Event
	 * @param class-string<T> $eventType The event type class to listen for.
	 * @param callable(T $event): (void|bool) $eventListener The event listener. If false is returned, stop event propagation.
	 * @return void
	 * @throws InvalidTypeException
	 */
	public function listen(string $eventType, callable $eventListener): void
	{
		$this->getEventsManager()->addListener($eventType, $eventListener);
	}
}
